Package: svMisc
Type: Package
Version: 1.4.3
Title: Miscellaneous Functions for 'SciViews::R'
Description: Functions required for the 'SciViews::R' dialect or for general
  use: manage a temporary environment attached to the search path, define
  synonyms for R functions using aka(), test if 'Aqua', 'Mac', 'Win' ... Show
  progress bar, etc.
Authors@R: c(
  person("Philippe", "Grosjean", role = c("aut", "cre"),
    email = "phgrosjean@sciviews.org",
    comment = c(ORCID = "0000-0002-2694-9471")),
  person("Romain", "Francois", role = "ctb",
    email = "romain@r-enthusiasts.com"),
  person("Kamil", "Barton", role = "ctb",
    email = "kamil.barton@uni-wuerzburg.de"))
Maintainer: Philippe Grosjean <phgrosjean@sciviews.org>
Depends: R (>= 4.2.0)
Imports: cli (>= 3.6.1), methods (>= 4.2.0), rlang (>= 1.1.1), stats
        (>= 4.2.0), tools (>= 4.2.0), utils (>= 4.2.0)
Suggests: tcltk (>= 4.2.0), knitr (>= 1.42), rmarkdown (>= 2.21),
        spelling (>= 2.2.1), testthat (>= 3.0.0)
License: GPL-2
URL: https://github.com/SciViews/svMisc,
        https://www.sciviews.org/svMisc/
BugReports: https://github.com/SciViews/svMisc/issues
RoxygenNote: 7.3.1
VignetteBuilder: knitr
Encoding: UTF-8
Language: en-US
ByteCompile: yes
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2024-11-14 11:18:38 UTC; phgrosjean
Author: Philippe Grosjean [aut, cre] (<https://orcid.org/0000-0002-2694-9471>),
  Romain Francois [ctb],
  Kamil Barton [ctb]
Repository: CRAN
Date/Publication: 2024-11-14 15:30:05 UTC
Built: R 4.6.0; ; 2026-04-25 17:58:32 UTC; unix
