# Generated by roxygen2: do not edit by hand

S3method("$",subsettable_type)
S3method("$",subsettable_which)
S3method(print,aka)
S3method(print,objList)
S3method(print,rbenchmark)
S3method(print,runnable)
S3method(print,section)
S3method(str,aka)
S3method(str,section)
export("?")
export(Install)
export(TempEnv)
export(about)
export(addActions)
export(addIcons)
export(addItems)
export(addMethods)
export(addTemp)
export(add_actions)
export(add_icons)
export(add_items)
export(add_methods)
export(add_temp)
export(aka)
export(argsTip)
export(args_tip)
export(assignTemp)
export(assign_temp)
export(batch)
export(callTip)
export(call_tip)
export(captureAll)
export(capture_all)
export(changeTemp)
export(change_temp)
export(compareRVersion)
export(compare_r_version)
export(completion)
export(cut_quantile)
export(def)
export(delete_temp)
export(descArgs)
export(descFun)
export(describe_args)
export(describe_function)
export(evalRjson)
export(eval_rjson)
export(ex)
export(existsTemp)
export(exists_temp)
export(fileEdit)
export(file_edit)
export(getTemp)
export(get_actions)
export(get_section)
export(get_temp)
export(guiCmd)
export(guiExport)
export(guiImport)
export(guiLoad)
export(guiReport)
export(guiSave)
export(guiSetwd)
export(guiSource)
export(gui_cmd)
export(gui_export)
export(gui_import)
export(gui_load)
export(gui_report)
export(gui_save)
export(gui_setwd)
export(gui_source)
export(helpSearchWeb)
export(isAqua)
export(isHelp)
export(isJGR)
export(isMac)
export(isRgui)
export(isSDI)
export(isWin)
export(is_aqua)
export(is_help)
export(is_jgr)
export(is_mac)
export(is_rgui)
export(is_rstudio)
export(is_rstudio_desktop)
export(is_rstudio_server)
export(is_sdi)
export(is_win)
export(listMethods)
export(listToJson)
export(listTypes)
export(list_methods)
export(list_to_json)
export(list_types)
export(objBrowse)
export(objClear)
export(objDir)
export(objInfo)
export(objList)
export(objMenu)
export(objSearch)
export(obj_browse)
export(obj_clear)
export(obj_dir)
export(obj_info)
export(obj_list)
export(obj_menu)
export(obj_search)
export(package)
export(parseText)
export(parse_text)
export(pcloud)
export(pcloud_crypto)
export(pkgManDescribe)
export(pkgManDetach)
export(pkgManGetAvailable)
export(pkgManGetInstalled)
export(pkgManGetMirrors)
export(pkgManInstall)
export(pkgManLoad)
export(pkgManRemove)
export(pkgManSetCRANMirror)
export(pkgman_describe)
export(pkgman_detach)
export(pkgman_get_available)
export(pkgman_get_installed)
export(pkgman_get_mirrors)
export(pkgman_install)
export(pkgman_load)
export(pkgman_remove)
export(pkgman_set_cran_mirror)
export(progress)
export(rbenchmark)
export(rmTemp)
export(rm_temp)
export(search_web)
export(section)
export(sourceClipboard)
export(source_clipboard)
export(systemDir)
export(systemFile)
export(system_dir)
export(system_file)
export(temp_env)
export(temp_var)
export(tempvar)
export(toRjson)
export(to_rjson)
export(warnings2)
export(write.objList)
importFrom(methods,existsFunction)
importFrom(methods,findFunction)
importFrom(methods,isGeneric)
importFrom(methods,new)
importFrom(methods,showMethods)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(rlang,is_interactive)
importFrom(stats,cor)
importFrom(stats,fft)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(tools,file_path_as_absolute)
importFrom(utils,"?")
importFrom(utils,RSiteSearch)
importFrom(utils,apropos)
importFrom(utils,available.packages)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,compareVersion)
importFrom(utils,file.edit)
importFrom(utils,file_test)
importFrom(utils,find)
importFrom(utils,flush.console)
importFrom(utils,getCRANmirrors)
importFrom(utils,getS3method)
importFrom(utils,help)
importFrom(utils,help.search)
importFrom(utils,install.packages)
importFrom(utils,installed.packages)
importFrom(utils,methods)
importFrom(utils,object.size)
importFrom(utils,packageDescription)
importFrom(utils,remove.packages)
importFrom(utils,setTxtProgressBar)
importFrom(utils,str)
importFrom(utils,tail)
importFrom(utils,txtProgressBar)
importFrom(utils,write.table)
